@@ -1,21 +1,21 @@
 //=============================================================================
 //   This file is part of VTKEdge. See vtkedge.org for more information.
 //
-//   Copyright (c) 2008 Kitware, Inc.
+//   Copyright (c) 2010 Kitware, Inc.
 //
-//   VTKEdge may be used under the terms of the GNU General Public License 
-//   version 3 as published by the Free Software Foundation and appearing in 
-//   the file LICENSE.txt included in the top level directory of this source
-//   code distribution. Alternatively you may (at your option) use any later 
-//   version of the GNU General Public License if such license has been 
-//   publicly approved by Kitware, Inc. (or its successors, if any).
+//   VTKEdge may be used under the terms of the BSD License
+//   Please see the file Copyright.txt in the root directory of
+//   VTKEdge for further information.
 //
-//   VTKEdge is distributed "AS IS" with NO WARRANTY OF ANY KIND, INCLUDING
-//   THE WARRANTIES OF DESIGN, MERCHANTABILITY, AND FITNESS FOR A PARTICULAR
-//   PURPOSE. See LICENSE.txt for additional details.
+//   Alternatively, you may see:
+//
+//   http://www.vtkedge.org/vtkedge/project/license.html
+//
+//
+//   For custom extensions, consulting services, or training for
+//   this or any other Kitware supported open source project, please
+//   contact Kitware at sales@kitware.com.
 //
-//   VTKEdge is available under alternative license terms. Please visit
-//   vtkedge.org or contact us at kitware@kitware.com for further information.
 //
 //=============================================================================
 #include "vtkKWEPaintbrushDrawing.h"
@@ -37,7 +37,7 @@
 #include <vtkstd/algorithm>
 #include <limits>
 
-vtkCxxRevisionMacro(vtkKWEPaintbrushDrawing, "$Revision: 725 $");
+vtkCxxRevisionMacro(vtkKWEPaintbrushDrawing, "$Revision: 3417 $");
 vtkStandardNewMacro(vtkKWEPaintbrushDrawing);
 
 //----------------------------------------------------------------------
@@ -77,7 +77,7 @@ void vtkKWEPaintbrushDrawing::SetImageData( vtkImageData * i)
 {
   vtkKWEPaintbrushSketch *a;
   vtkCollectionSimpleIterator ait;
-  for ( this->Collection->InitTraversal(ait); 
+  for ( this->Collection->InitTraversal(ait);
     (a=static_cast< vtkKWEPaintbrushSketch * >(this->Collection->GetNextItemAsObject(ait))); )
     {
     a->SetImageData(i);
@@ -92,7 +92,7 @@ void vtkKWEPaintbrushDrawing::InitializeData()
     {
     this->PaintbrushData->Delete();
     }
-  
+
   if (this->Representation == vtkKWEPaintbrushEnums::Binary)
     {
     this->PaintbrushData = vtkKWEPaintbrushStencilData::New();
@@ -106,13 +106,13 @@ void vtkKWEPaintbrushDrawing::InitializeData()
     this->PaintbrushData = vtkKWEPaintbrushLabelData::New();
     }
 
-  this->AllocatePaintbrushData();  
+  this->AllocatePaintbrushData();
   if (this->GetNumberOfItems() == 0)
     {
     // If there are no items, create one by default. The user, when he creates
-    // this widget, expects there to be at least a sketch alive, so he can 
+    // this widget, expects there to be at least a sketch alive, so he can
     // sketch something.
-    
+
     vtkKWEPaintbrushSketch *m = vtkKWEPaintbrushSketch::New();
     m->SetRepresentation(this->Representation);
     this->AddItem(m);
@@ -120,28 +120,28 @@ void vtkKWEPaintbrushDrawing::InitializeData()
 
     this->AssignUniqueLabelToSketch(m); // Assign a unique label to the sketch.
     }
-  
+
   else
-    { 
+    {
 
     vtkKWEPaintbrushSketch *a;
     vtkCollectionSimpleIterator ait;
 
-    // Initialize all the sketches in the drawing. Be prepared to lose your 
+    // Initialize all the sketches in the drawing. Be prepared to lose your
     // sketches.
-  
-    for ( this->Collection->InitTraversal(ait); 
+
+    for ( this->Collection->InitTraversal(ait);
       (a=static_cast< vtkKWEPaintbrushSketch * >(this->Collection->GetNextItemAsObject(ait))); )
       {
       a->Initialize();
       }
-      
-  
+
+
     if (this->Representation == vtkKWEPaintbrushEnums::Label)
       {
       // For Label maps, all sketches point to the same label map, since they
       // will be editing different labels within the same label map.
-      this->SetPaintbrushData( static_cast< 
+      this->SetPaintbrushData( static_cast<
           vtkKWEPaintbrushLabelData * >(this->PaintbrushData) );
       }
     }
@@ -153,7 +153,7 @@ SetPaintbrushOperation( vtkKWEPaintbrushOperation * i)
 {
   vtkKWEPaintbrushSketch *a;
   vtkCollectionSimpleIterator ait;
-  for ( this->Collection->InitTraversal(ait); 
+  for ( this->Collection->InitTraversal(ait);
     (a=static_cast< vtkKWEPaintbrushSketch * >(this->Collection->GetNextItemAsObject(ait))); )
     {
     a->SetPaintbrushOperation(i);
@@ -173,25 +173,25 @@ void vtkKWEPaintbrushDrawing::AllocatePaintbrushData()
     {
     vtkErrorMacro( << "Must call Initialize() before using this class !");
     }
-  
+
   int extent[6];
   double spacing[3], origin[3];
   i->GetWholeExtent(extent);
   i->GetSpacing(spacing);
   i->GetOrigin(origin);
 
-  // If a label map has been supplied (and our representation is a label map), 
-  // used the supplied one, otherwise allocate fresh paintbrush data with the 
+  // If a label map has been supplied (and our representation is a label map),
+  // used the supplied one, otherwise allocate fresh paintbrush data with the
   // same size as the canvas.
-  
+
   this->PaintbrushData->SetExtent(extent);
   this->PaintbrushData->SetSpacing(spacing);
   this->PaintbrushData->SetOrigin(origin);
   if (this->Representation == vtkKWEPaintbrushEnums::Label && this->LabelMap )
     {
-    
-    // Sanity check : Ensure that the label map supplied by the user has the 
-    // same extents as the canvas, or we will have cryptic errors later on 
+
+    // Sanity check : Ensure that the label map supplied by the user has the
+    // same extents as the canvas, or we will have cryptic errors later on
     // during the overlay-blend render phase.
     if (!vtkKWEPaintbrushUtilities::ExtentIsEqualToExtent(
                      extent, this->LabelMap->GetExtent()))
@@ -210,7 +210,7 @@ void vtkKWEPaintbrushDrawing::AllocatePaintbrushData()
 }
 
 //----------------------------------------------------------------------
-int vtkKWEPaintbrushDrawing::AddNewStroke( 
+int vtkKWEPaintbrushDrawing::AddNewStroke(
       int n,    // Stroke is added to the n'th sketch
       int type, // Draw or Erase ?
       vtkKWEPaintbrushData *userSpecifiedStrokeData // User provided initialization
@@ -228,7 +228,7 @@ void vtkKWEPaintbrushDrawing::AddShapeToCurrentStroke( int n, double p[3] )
 {
   if (n < this->GetNumberOfItems() && n >= 0)
     {
-    // Add (or subtract) a shape centered at 'p' to the current stroke. 
+    // Add (or subtract) a shape centered at 'p' to the current stroke.
     // Also add (or subtract) the same to this->PaintbrushData, as a service.
     this->GetItem(n)->AddShapeToCurrentStroke(p, this->PaintbrushData);
     }
@@ -238,13 +238,13 @@ void vtkKWEPaintbrushDrawing::AddShapeToCurrentStroke( int n, double p[3] )
 void vtkKWEPaintbrushDrawing::SetPaintbrushData( vtkKWEPaintbrushLabelData * data )
 {
   vtkSetObjectBodyMacro( PaintbrushData, vtkKWEPaintbrushData, data );
-  
+
   // For label maps, all sketches share the same data, this label map.
   // Set it on all of them.
 
   vtkKWEPaintbrushSketch *a;
   vtkCollectionSimpleIterator ait;
-  for ( this->Collection->InitTraversal(ait); 
+  for ( this->Collection->InitTraversal(ait);
     (a=static_cast< vtkKWEPaintbrushSketch * >(
       this->Collection->GetNextItemAsObject(ait))); )
     {
@@ -254,7 +254,7 @@ void vtkKWEPaintbrushDrawing::SetPaintbrushData( vtkKWEPaintbrushLabelData * dat
   // Sanity check. Make sure the scalar type of the data is correct.
   if (data && data->GetLabelMap())
     {
-    if (data->GetLabelMap()->GetScalarType() 
+    if (data->GetLabelMap()->GetScalarType()
               != vtkKWEPaintbrushEnums::GetLabelType())
       {
       vtkErrorMacro( << "DataType of the label map must match the enumeration in "
@@ -285,7 +285,7 @@ unsigned long vtkKWEPaintbrushDrawing::GetMTime()
   unsigned long result = 0;
   vtkKWEPaintbrushSketch *a;
   vtkCollectionSimpleIterator ait;
-  for ( this->Collection->InitTraversal(ait); 
+  for ( this->Collection->InitTraversal(ait);
           (a=static_cast< vtkKWEPaintbrushSketch * >(
             this->Collection->GetNextItemAsObject(ait))); )
     {
@@ -294,14 +294,14 @@ unsigned long vtkKWEPaintbrushDrawing::GetMTime()
     }
 
   unsigned long mtime = vtkObject::GetMTime();
-  result = ( mtime > result ? mtime : result);  
+  result = ( mtime > result ? mtime : result);
   return result;
 }
 
 //----------------------------------------------------------------------------
-vtkKWEPaintbrushSketch * vtkKWEPaintbrushDrawing::AddItem() 
+vtkKWEPaintbrushSketch * vtkKWEPaintbrushDrawing::AddItem()
 {
-  if (this->MaximumNumberOfSketches && 
+  if (this->MaximumNumberOfSketches &&
       this->GetNumberOfItems() == this->MaximumNumberOfSketches)
     {
     return NULL; // We hit the limit.
@@ -327,9 +327,9 @@ vtkKWEPaintbrushSketch * vtkKWEPaintbrushDrawing::AddItem()
 }
 
 //----------------------------------------------------------------------------
-void vtkKWEPaintbrushDrawing::AddItem(vtkKWEPaintbrushSketch *a) 
+void vtkKWEPaintbrushDrawing::AddItem(vtkKWEPaintbrushSketch *a)
 {
-  if (this->MaximumNumberOfSketches && 
+  if (this->MaximumNumberOfSketches &&
       this->GetNumberOfItems() == this->MaximumNumberOfSketches)
     {
     return; // We hit the limit.
@@ -357,6 +357,23 @@ vtkKWEPaintbrushSketch *vtkKWEPaintbrushDrawing::GetItem(int i)
           this->Collection->GetItemAsObject(i));
 }
 
+//----------------------------------------------------------------------------
+vtkKWEPaintbrushSketch * vtkKWEPaintbrushDrawing::
+GetItemWithLabel( vtkKWEPaintbrushEnums::LabelType l )
+{
+  vtkKWEPaintbrushSketch *a;
+  vtkCollectionSimpleIterator ait;
+  for ( this->Collection->InitTraversal(ait);
+          (a=static_cast< vtkKWEPaintbrushSketch * >(
+            this->Collection->GetNextItemAsObject(ait))); )
+    {
+    if (a->GetLabel() == l)
+      {
+      return a;
+      }
+    }
+  return NULL;
+}
 
 //----------------------------------------------------------------------------
 int vtkKWEPaintbrushDrawing::IsItemPresent( vtkKWEPaintbrushSketch * s )
@@ -364,6 +381,24 @@ int vtkKWEPaintbrushDrawing::IsItemPresent( vtkKWEPaintbrushSketch * s )
   return this->Collection->IsItemPresent(s);
 }
 
+//----------------------------------------------------------------------------
+int vtkKWEPaintbrushDrawing::GetIndexOfItem( vtkKWEPaintbrushSketch * s )
+{
+  int result = 0;
+  vtkKWEPaintbrushSketch *a;
+  vtkCollectionSimpleIterator ait;
+  for ( this->Collection->InitTraversal(ait);
+          (a=static_cast< vtkKWEPaintbrushSketch * >(
+            this->Collection->GetNextItemAsObject(ait))); result++ )
+    {
+    if (s == a)
+      {
+      return result;
+      }
+    }
+  return -1;
+}
+
 //----------------------------------------------------------------------------
 void vtkKWEPaintbrushDrawing::RemoveAllItems()
 {
@@ -384,7 +419,7 @@ int vtkKWEPaintbrushDrawing::RemoveItem( vtkKWEPaintbrushSketch *s )
 //----------------------------------------------------------------------------
 void vtkKWEPaintbrushDrawing::ShallowCopy( vtkDataObject * src )
 {
-  if (src == this) 
+  if (src == this)
     {
     return;
     }
@@ -423,7 +458,7 @@ void vtkKWEPaintbrushDrawing::ComposeStrokes()
 
   vtkKWEPaintbrushSketch *sketch;
   vtkCollectionSimpleIterator it;
-  for ( this->Collection->InitTraversal(it); 
+  for ( this->Collection->InitTraversal(it);
          (sketch=static_cast< vtkKWEPaintbrushSketch * >(
            this->Collection->GetNextItemAsObject(it))); )
     {
@@ -434,7 +469,7 @@ void vtkKWEPaintbrushDrawing::ComposeStrokes()
   vtkstd::sort( strokes.begin(), strokes.end(), vtkKWEPaintbrushStroke::IsRecent );
 
   // FIXME
-  // We could be smarter by composing just the strokes with extents that 
+  // We could be smarter by composing just the strokes with extents that
   // intersect with the one just undone/redone.
 
   vtkstd::vector< vtkKWEPaintbrushStroke * >::iterator sit = strokes.begin();
@@ -443,9 +478,9 @@ void vtkKWEPaintbrushDrawing::ComposeStrokes()
     vtkKWEPaintbrushStroke * stroke = *sit;
     if (stroke->GetState() == vtkKWEPaintbrushEnums::Erase)
       {
-      this->PaintbrushData->Subtract(stroke->GetPaintbrushData()); 
+      this->PaintbrushData->Subtract(stroke->GetPaintbrushData());
       }
-    else   
+    else
       {
       this->PaintbrushData->Add(stroke->GetPaintbrushData());
       }
@@ -461,8 +496,8 @@ void vtkKWEPaintbrushDrawing::AssignUniqueLabelToSketch( vtkKWEPaintbrushSketch
     {
     labels[i] = this->GetItem(i)->GetLabel();
     }
-  vtkKWEPaintbrushEnums::LabelType minLabel = 1; 
-  vtkKWEPaintbrushEnums::LabelType maxLabel 
+  vtkKWEPaintbrushEnums::LabelType minLabel = 1;
+  vtkKWEPaintbrushEnums::LabelType maxLabel
     = vtkstd::numeric_limits< vtkKWEPaintbrushEnums::LabelType >::max();
 
   for (vtkKWEPaintbrushEnums::LabelType i = minLabel; i < maxLabel; i++)
@@ -470,7 +505,7 @@ void vtkKWEPaintbrushDrawing::AssignUniqueLabelToSketch( vtkKWEPaintbrushSketch
     if (std::find(labels.begin(), labels.end(), i) != labels.end())
       {
       s->SetLabel(i);
-      return;  
+      return;
       }
     }
 }
@@ -478,13 +513,13 @@ void vtkKWEPaintbrushDrawing::AssignUniqueLabelToSketch( vtkKWEPaintbrushSketch
 //----------------------------------------------------------------------------
 void vtkKWEPaintbrushDrawing::InitializeSketches()
 {
-  // For representations that are not label maps, this should already have 
+  // For representations that are not label maps, this should already have
   // been done.
-  
+
   if (!this->PaintbrushData)
     {
     vtkErrorMacro( << "PaintbrushData is NULL. Did you add this sketch to the "
-      << "drawing and call InitializeData on the drawing prior to " 
+      << "drawing and call InitializeData on the drawing prior to "
       << "initializing the sketch ?");
     return;
     }
@@ -498,38 +533,118 @@ void vtkKWEPaintbrushDrawing::InitializeSketches()
 
   if (this->Representation != vtkKWEPaintbrushEnums::Label)
     {
-    vtkErrorMacro( << "This method is only intended to be used when editing LabelMaps." 
+    vtkErrorMacro( << "This method is only intended to be used when editing LabelMaps."
        << " Did you set the representation first ?" );
     return;
     }
-  
-  vtkstd::vector< vtkKWEPaintbrushEnums::LabelType > 
+
+  vtkstd::vector< vtkKWEPaintbrushEnums::LabelType >
         labels(this->Collection->GetNumberOfItems());
   unsigned int i = 0;
   vtkCollectionSimpleIterator ait;
   vtkKWEPaintbrushSketch *a;
-  for ( this->Collection->InitTraversal(ait); 
+  for ( this->Collection->InitTraversal(ait);
       (a=static_cast< vtkKWEPaintbrushSketch * >(this->Collection->GetNextItemAsObject(ait))); )
-    { 
+    {
     labels[i++] = a->GetLabel();
     }
 
-  vtkImageData *image = vtkKWEPaintbrushLabelData::SafeDownCast( 
+  vtkImageData *image = vtkKWEPaintbrushLabelData::SafeDownCast(
                         this->PaintbrushData )->GetLabelMap();
 
   // Populate all the strokes at one shot
-  vtkstd::map< vtkKWEPaintbrushEnums::LabelType, 
-                 vtkSmartPointer< vtkKWEPaintbrushStencilData > > initialStrokes 
+  vtkstd::map< vtkKWEPaintbrushEnums::LabelType,
+                 vtkSmartPointer< vtkKWEPaintbrushStencilData > > initialStrokes
       =  vtkKWEPaintbrushUtilities::GetStencilsFromImage( image, labels );
-  
+
   // Initialize all the sketches with their user-specified strokes.
-  for ( this->Collection->InitTraversal(ait); 
+  for ( this->Collection->InitTraversal(ait);
       (a=static_cast< vtkKWEPaintbrushSketch * >(this->Collection->GetNextItemAsObject(ait))); )
-    { 
+    {
     a->AddNewStroke( vtkKWEPaintbrushEnums::Draw, initialStrokes[a->GetLabel()], NULL, true );
     }
 }
 
+//----------------------------------------------------------------------------
+void vtkKWEPaintbrushDrawing::CreateSketches()
+{
+  // For representations that are not label maps, this should already have
+  // been done.
+
+  if (!this->PaintbrushData)
+    {
+    vtkErrorMacro( << "PaintbrushData is NULL. Did you add this sketch to the "
+      << "drawing and call InitializeData on the drawing prior to "
+      << "initializing the sketch ?");
+    return;
+    }
+
+  if (this->Representation != vtkKWEPaintbrushEnums::Label)
+    {
+    vtkErrorMacro( << "This method is only intended to be used when editing LabelMaps."
+       << " Did you set the representation first ?" );
+    return;
+    }
+
+
+  vtkKWEPaintbrushLabelData *paintbrushLabelData =
+    vtkKWEPaintbrushLabelData::SafeDownCast(this->PaintbrushData);
+  typedef vtkKWEPaintbrushLabelData::LabelSetType LabelSetType;
+  LabelSetType labelSet = paintbrushLabelData->GetLabels();
+
+  // fast path if this is the first time we are initilizing. This enables us to
+  // initialize all sketches at one shot.
+  bool firstTime = this->GetNumberOfItems() == 0 ? true : false;
+
+  // Create new sketches for each of the labels in the label map, taking care
+  // check if we don't have a sketch with that label already.
+  for ( LabelSetType::const_iterator it = labelSet.begin();
+        it != labelSet.end(); ++it )
+    {
+    vtkKWEPaintbrushEnums::LabelType label = *it;
+    if (!this->GetItemWithLabel(label))
+      {
+      vtkKWEPaintbrushSketch * sketch = this->AddItem();
+      sketch->SetLabel(label);
+      }
+    }
+
+  // Remove any sketches that aren't in this label map
+  if (!firstTime)
+    {
+    std::vector< vtkKWEPaintbrushSketch * > toRemove;
+    const int nSketches = this->GetNumberOfItems();
+    for (int i = 0; i < nSketches; i++)
+      {
+      vtkKWEPaintbrushSketch *s = this->GetItem(i);
+      if (labelSet.find(s->GetLabel()) == labelSet.end())
+        {
+        toRemove.push_back(s);
+        }
+      }
+
+    const int nToRemove = toRemove.size();
+    for (int i = 0; i < nToRemove; i++)
+      {
+      this->RemoveItem(toRemove[i]);
+      }
+    }
+
+  // Now that we've created these sketches, let's initialize all the sketches
+  // based on the label map.
+  this->InitializeSketches();
+}
+
+//----------------------------------------------------------------------------
+void vtkKWEPaintbrushDrawing::CollapseHistory()
+{
+  const int nSketches = this->GetNumberOfItems();
+  for (int i = 0; i < nSketches; i++)
+    {
+    this->GetItem(i)->CollapseHistory();
+    }
+}
+
 //----------------------------------------------------------------------------
 void vtkKWEPaintbrushDrawing::PrintSelf(ostream& os, vtkIndent indent)
 {
@@ -554,7 +669,7 @@ void vtkKWEPaintbrushDrawing::PrintSelf(ostream& os, vtkIndent indent)
     }
   if (this->MaximumNumberOfSketches)
     {
-    os << indent << "MaximumNumberOfSketches: " 
+    os << indent << "MaximumNumberOfSketches: "
        << this->MaximumNumberOfSketches << endl;
     }
   this->Superclass::PrintSelf(os,indent);
