@@ -1,21 +1,21 @@
 //=============================================================================
 //   This file is part of VTKEdge. See vtkedge.org for more information.
 //
-//   Copyright (c) 2008 Kitware, Inc.
+//   Copyright (c) 2010 Kitware, Inc.
 //
-//   VTKEdge may be used under the terms of the GNU General Public License 
-//   version 3 as published by the Free Software Foundation and appearing in 
-//   the file LICENSE.txt included in the top level directory of this source
-//   code distribution. Alternatively you may (at your option) use any later 
-//   version of the GNU General Public License if such license has been 
-//   publicly approved by Kitware, Inc. (or its successors, if any).
+//   VTKEdge may be used under the terms of the BSD License
+//   Please see the file Copyright.txt in the root directory of
+//   VTKEdge for further information.
 //
-//   VTKEdge is distributed "AS IS" with NO WARRANTY OF ANY KIND, INCLUDING
-//   THE WARRANTIES OF DESIGN, MERCHANTABILITY, AND FITNESS FOR A PARTICULAR
-//   PURPOSE. See LICENSE.txt for additional details.
+//   Alternatively, you may see:
+//
+//   http://www.vtkedge.org/vtkedge/project/license.html
+//
+//
+//   For custom extensions, consulting services, or training for
+//   this or any other Kitware supported open source project, please
+//   contact Kitware at sales@kitware.com.
 //
-//   VTKEdge is available under alternative license terms. Please visit
-//   vtkedge.org or contact us at kitware@kitware.com for further information.
 //
 //=============================================================================
 #include "vtkKWEPaintbrushLabelData.h"
@@ -31,12 +31,14 @@
 #include "vtkDataArray.h"
 #include "vtkObjectFactory.h"
 #include "vtkMath.h"
+#include "vtkImageShiftScale.h"
+#include "vtkImageCast.h"
 
 #include <math.h>
+#include <limits>
 
-vtkCxxRevisionMacro(vtkKWEPaintbrushLabelData, "$Revision: 763 $");
+vtkCxxRevisionMacro(vtkKWEPaintbrushLabelData, "$Revision: 3550 $");
 vtkStandardNewMacro(vtkKWEPaintbrushLabelData);
-vtkCxxSetObjectMacro(vtkKWEPaintbrushLabelData, LabelMap, vtkImageData);
 
 //----------------------------------------------------------------------------
 vtkKWEPaintbrushEnums::LabelType vtkKWEPaintbrushLabelData::NoLabelValue = 0;
@@ -48,7 +50,7 @@ vtkKWEPaintbrushLabelData::vtkKWEPaintbrushLabelData()
   this->LabelMap->SetScalarType( vtkKWEPaintbrushEnums::GetLabelType() );
 
   this->Information->Set( vtkDataObject::DATA_EXTENT_TYPE(), VTK_3D_EXTENT );
-  this->Information->Set( vtkDataObject::DATA_EXTENT(), 
+  this->Information->Set( vtkDataObject::DATA_EXTENT(),
                           this->LabelMap->GetExtent(), 6);
 }
 
@@ -74,7 +76,7 @@ void vtkKWEPaintbrushLabelData::CopyInformationToPipeline(vtkInformation* reques
                                                     vtkInformation* output,
                                                     int forceCopy)
 {
-  this->LabelMap->CopyInformationToPipeline( 
+  this->LabelMap->CopyInformationToPipeline(
           request, input, output, forceCopy );
 }
 
@@ -155,9 +157,9 @@ void vtkKWEPaintbrushLabelData::Allocate( double value )
   this->LabelMap->AllocateScalars();
 
   // Fill with the unused label value.
-  
+
   vtkDataArray * array = this->LabelMap->GetPointData()->GetScalars();
-  vtkKWEPaintbrushEnums::LabelType *arrayPointer = 
+  vtkKWEPaintbrushEnums::LabelType *arrayPointer =
     static_cast<vtkKWEPaintbrushEnums::LabelType *>(array->GetVoidPointer(0));
   unsigned long size = array->GetDataSize();
 
@@ -179,9 +181,9 @@ void vtkKWEPaintbrushLabelData::Clear( vtkKWEPaintbrushEnums::LabelType label )
   else
     {
     // Clear label
-  
+
     vtkDataArray * array = this->LabelMap->GetPointData()->GetScalars();
-    vtkKWEPaintbrushEnums::LabelType *arrayPointer = 
+    vtkKWEPaintbrushEnums::LabelType *arrayPointer =
       static_cast<vtkKWEPaintbrushEnums::LabelType *>(array->GetVoidPointer(0));
     unsigned long size = array->GetDataSize();
 
@@ -219,42 +221,42 @@ int vtkKWEPaintbrushLabelData::Add( vtkKWEPaintbrushData * data, bool forceMutab
   data->GetExtent(extentToBeAdded);
   this->GetExtent(currentExtent);
 
-  if (!vtkKWEPaintbrushUtilities::GetIntersectingExtents( 
+  if (!vtkKWEPaintbrushUtilities::GetIntersectingExtents(
              currentExtent, extentToBeAdded, extent ))
     {
     // The stuff we are adding is entirely outside our bounds.
-    // Nothing to add. 
-    return 0; 
+    // Nothing to add.
+    return 0;
     }
 
-  // Handle Immubtability of sketches. 
+  // Handle Immubtability of sketches.
   // For details, see vtkKWEPaintbrushProperty::SetMubtale
   const bool mutableLabelsPresent = (this->ImmutableLabels.size() > 0);
-  vtkstd::set< vtkKWEPaintbrushEnums::LabelType >::const_iterator 
+  vtkstd::set< vtkKWEPaintbrushEnums::LabelType >::const_iterator
                     iterEnd = this->ImmutableLabels.end();
 
-  
-  if (vtkKWEPaintbrushLabelData *labelData = 
+
+  if (vtkKWEPaintbrushLabelData *labelData =
       vtkKWEPaintbrushLabelData::SafeDownCast(data))
     {
     // PaintbrushLabelData + PaintbrushLabelData
 
-    vtkImageIterator< vtkKWEPaintbrushEnums::LabelType > it2( 
+    vtkImageIterator< vtkKWEPaintbrushEnums::LabelType > it2(
                     labelData->GetLabelMap(), extent );
-    vtkImageIterator< vtkKWEPaintbrushEnums::LabelType > it1( 
+    vtkImageIterator< vtkKWEPaintbrushEnums::LabelType > it1(
                     this->LabelMap, extent );
 
     bool immutableLabel = false;
-    vtkKWEPaintbrushEnums::LabelType lastLabel = 
+    vtkKWEPaintbrushEnums::LabelType lastLabel =
                                   vtkKWEPaintbrushLabelData::NoLabelValue;
     while( !it1.IsAtEnd() )
-      { 
-      vtkKWEPaintbrushEnums::LabelType *inSI    = it1.BeginSpan();  
-      vtkKWEPaintbrushEnums::LabelType *inSI2   = it2.BeginSpan();  
+      {
+      vtkKWEPaintbrushEnums::LabelType *inSI    = it1.BeginSpan();
+      vtkKWEPaintbrushEnums::LabelType *inSI2   = it2.BeginSpan();
       vtkKWEPaintbrushEnums::LabelType *inSIEnd = it1.EndSpan();
       if( mutableLabelsPresent && !forceMutable )
         {
-        while (inSI != inSIEnd) 
+        while (inSI != inSIEnd)
           {
           vtkKWEPaintbrushEnums::LabelType l = *inSI2;
           if (l != vtkKWEPaintbrushLabelData::NoLabelValue)
@@ -278,11 +280,11 @@ int vtkKWEPaintbrushLabelData::Add( vtkKWEPaintbrushData * data, bool forceMutab
             }
           ++inSI;
           ++inSI2;
-          }      
+          }
         }
       else
         {
-        while (inSI != inSIEnd) 
+        while (inSI != inSIEnd)
           {
           vtkKWEPaintbrushEnums::LabelType l = *inSI2;
           if (l != vtkKWEPaintbrushLabelData::NoLabelValue)
@@ -291,7 +293,7 @@ int vtkKWEPaintbrushLabelData::Add( vtkKWEPaintbrushData * data, bool forceMutab
             }
           ++inSI;
           ++inSI2;
-          }      
+          }
         }
       it1.NextSpan();
       it2.NextSpan();
@@ -301,71 +303,7 @@ int vtkKWEPaintbrushLabelData::Add( vtkKWEPaintbrushData * data, bool forceMutab
   else if (vtkKWEPaintbrushStencilData *binaryPaintbrushData =
            vtkKWEPaintbrushStencilData::SafeDownCast(data))
     {
-    // PaintbrushLabelData + PaintbrushStencilData
-      
-    vtkImageStencilData *stencilData 
-      = binaryPaintbrushData->GetImageStencilData();
-
-    vtkKWEPaintbrushEnums::LabelType l = binaryPaintbrushData->GetLabel();
-
-    int r1, r2, moreSubExtents, iter;
-    bool immutableLabel = false;
-    vtkKWEPaintbrushEnums::LabelType lastLabel = 
-                                  vtkKWEPaintbrushLabelData::NoLabelValue;
-    for (int z=extent[4]; z <= extent[5]; z++)
-      {
-      for (int y=extent[2]; y <= extent[3]; y++)
-        {
-        iter = 0;
-        moreSubExtents = 1;
-        while( moreSubExtents )
-          {
-          moreSubExtents = stencilData->GetNextExtent(
-            r1, r2, extent[0], extent[1], y, z, iter);
-
-          // sanity check
-          if (r1 <= r2)
-            {
-            vtkKWEPaintbrushEnums::LabelType *beginExtent =
-              static_cast<vtkKWEPaintbrushEnums::LabelType *>(this->LabelMap->GetScalarPointer(r1, y, z));
-            vtkKWEPaintbrushEnums::LabelType *endExtent   =
-              static_cast<vtkKWEPaintbrushEnums::LabelType *>(this->LabelMap->GetScalarPointer(r2, y, z));
-            if( mutableLabelsPresent && !forceMutable )
-              {
-              while (beginExtent <= endExtent)
-                {
-                // If this voxel is mutable..
-                if( lastLabel != *beginExtent )
-                  {
-                  lastLabel = *beginExtent;
-                  if( this->ImmutableLabels.find(*beginExtent) == iterEnd )
-                    {
-                    immutableLabel = false;
-                    }
-                  else
-                    {
-                    immutableLabel = true;
-                    }
-                  }
-                if( !immutableLabel )
-                  {
-                  *beginExtent = l;
-                  }
-                ++beginExtent;
-                }
-              }
-            else
-              {
-              while (beginExtent <= endExtent)
-                {
-                *beginExtent = l;
-                ++beginExtent;
-                }
-              }
-            }
-          } // end for each extent tuple
-        } // end for each scan line
-      } // end of each slice 
+    this->AddStencil( binaryPaintbrushData, forceMutable );
     }
 
   this->LabelMap->Modified();
@@ -373,6 +311,165 @@ int vtkKWEPaintbrushLabelData::Add( vtkKWEPaintbrushData * data, bool forceMutab
   return 1;
 }
 
+//----------------------------------------------------------------------------
+int vtkKWEPaintbrushLabelData::
+AddStencil( vtkKWEPaintbrushStencilData * data, bool forceMutable )
+{
+  int extentToBeAdded[6], currentExtent[6], extent[6];
+  data->GetExtent(extentToBeAdded);
+  this->GetExtent(currentExtent);
+
+  if (!vtkKWEPaintbrushUtilities::GetIntersectingExtents(
+             currentExtent, extentToBeAdded, extent ))
+    {
+    // The stuff we are adding is entirely outside our bounds.
+    // Nothing to add.
+    return 0;
+    }
+
+  // Handle Immubtability of sketches.
+  // For details, see vtkKWEPaintbrushProperty::SetMubtale
+  const bool mutableLabelsPresent = (this->ImmutableLabels.size() > 0);
+  vtkstd::set< vtkKWEPaintbrushEnums::LabelType >::const_iterator
+                    iterEnd = this->ImmutableLabels.end();
+  
+  // PaintbrushLabelData + PaintbrushStencilData
+
+  vtkImageStencilData *stencilData = data->GetImageStencilData();
+
+  // Compute incremental deltas for Undo-Redo, if requested.
+  vtkSmartPointer< vtkImageStencilData > tempData = stencilData;
+  if (this->ComputeDelta)
+    {
+    tempData = vtkSmartPointer< vtkImageStencilData >::New();
+    tempData->DeepCopy(stencilData);
+    }
+
+  vtkKWEPaintbrushEnums::LabelType l = data->GetLabel();
+
+  int r1, r2, moreSubExtents, iter;
+  bool immutableLabel = false;
+  vtkKWEPaintbrushEnums::LabelType lastLabel =
+                                vtkKWEPaintbrushLabelData::NoLabelValue;
+  for (int z=extent[4]; z <= extent[5]; z++)
+    {
+    for (int y=extent[2]; y <= extent[3]; y++)
+      {
+      iter = 0;
+      moreSubExtents = 1;
+      while( moreSubExtents )
+        {
+        moreSubExtents = tempData->GetNextExtent(
+          r1, r2, extent[0], extent[1], y, z, iter);
+
+        int removeStart = -1, removeEnd = -1;
+
+        // sanity check
+        if (r1 <= r2)
+          {
+          vtkKWEPaintbrushEnums::LabelType *beginExtent =
+            static_cast<vtkKWEPaintbrushEnums::LabelType *>(this->LabelMap->GetScalarPointer(r1, y, z));
+          vtkKWEPaintbrushEnums::LabelType *endExtent   =
+            static_cast<vtkKWEPaintbrushEnums::LabelType *>(this->LabelMap->GetScalarPointer(r2, y, z));
+          if( mutableLabelsPresent && !forceMutable )
+            {
+            while (beginExtent <= endExtent)
+              {
+              // If this voxel is mutable..
+              if( lastLabel != *beginExtent )
+                {
+                lastLabel = *beginExtent;
+                if( this->ImmutableLabels.find(*beginExtent) == iterEnd )
+                  {
+                  immutableLabel = false;
+                  }
+                else
+                  {
+                  immutableLabel = true;
+                  }
+                }
+              if( !immutableLabel )
+                {
+                if (*beginExtent != l)
+                  {
+                  *beginExtent = l;
+
+                  // This scan line already exists on the data. Remove it from
+                  // the stroke
+                  if (this->ComputeDelta && removeStart != -1 && removeEnd == -1 && removeStart < r1)
+                    {
+                    stencilData->RemoveExtent( removeStart, r1-1, y, z );
+                    removeStart = removeEnd = -1; // reset them.
+                    }
+                  }
+                else
+                  {
+                  if (removeStart == -1)
+                    {
+                    removeStart = r1;
+                    }
+                  }
+                }
+              else // this pixel is locked to any changes
+                {
+                if (removeStart == -1)
+                  {
+                  removeStart = r1;
+                  }
+                }
+              ++beginExtent;
+              ++r1;
+              }
+
+            // This scan line already exists on the data. Remove it from the
+            // stroke
+            if (this->ComputeDelta && removeStart != -1 && removeEnd == -1)
+              {
+              stencilData->RemoveExtent( removeStart, r1 - 1, y, z );
+              }
+
+            }
+          else
+            {
+            while (beginExtent <= endExtent)
+              {
+              if (*beginExtent != l)
+                {
+                *beginExtent = l;
+
+                // This scan line already exists on the data. Remove it from
+                // the stroke
+                if (this->ComputeDelta && removeStart != -1 && removeEnd == -1 && removeStart < r1)
+                  {
+                  stencilData->RemoveExtent( removeStart, r1-1, y, z );
+                  removeStart = removeEnd = -1; // reset them.
+                  }
+                }
+              else
+                {
+                if (removeStart == -1)
+                  {
+                  removeStart = r1;
+                  }
+                }
+              ++beginExtent;
+              ++r1;
+              }
+
+            // This scan line already exists on the data. Remove it from the
+            // stroke
+            if (this->ComputeDelta && removeStart != -1 && removeEnd == -1)
+              {
+              stencilData->RemoveExtent( removeStart, r1 - 1, y, z );
+              }
+
+            }
+          }
+        } // end for each extent tuple
+      } // end for each scan line
+    } // end of each slice
+}
+
 //----------------------------------------------------------------------------
 int vtkKWEPaintbrushLabelData::Subtract( vtkKWEPaintbrushData * data, bool forceMutable )
 {
@@ -380,39 +477,40 @@ int vtkKWEPaintbrushLabelData::Subtract( vtkKWEPaintbrushData * data, bool force
   data->GetExtent(extentToBeSubtracted);
   this->GetExtent(currentExtent);
 
-  if (!vtkKWEPaintbrushUtilities::GetIntersectingExtents( 
+  if (!vtkKWEPaintbrushUtilities::GetIntersectingExtents(
          currentExtent, extentToBeSubtracted, extent ))
     {
     // The stuff we are removing is entirely outside our bounds.
-    // Nothing to add. 
-    return 0; 
+    // Nothing to add.
+    return 0;
     }
 
-  // Handle Immubtability of sketches. 
+  // Handle Immubtability of sketches.
   // For details, see vtkKWEPaintbrushProperty::SetMubtale
   const bool mutableLabelsPresent = (this->ImmutableLabels.size() > 0);
-  vtkstd::set< vtkKWEPaintbrushEnums::LabelType >::const_iterator 
+  vtkstd::set< vtkKWEPaintbrushEnums::LabelType >::const_iterator
                     iterEnd = this->ImmutableLabels.end();
-  
 
-  if (vtkKWEPaintbrushLabelData *labelData = 
+
+  if (vtkKWEPaintbrushLabelData *labelData =
       vtkKWEPaintbrushLabelData::SafeDownCast(data))
     {
-    vtkImageIterator< vtkKWEPaintbrushEnums::LabelType > it2( 
+  
+    vtkImageIterator< vtkKWEPaintbrushEnums::LabelType > it2(
                     labelData->GetLabelMap(), extent );
-    vtkImageIterator< vtkKWEPaintbrushEnums::LabelType > it1( 
+    vtkImageIterator< vtkKWEPaintbrushEnums::LabelType > it1(
                     this->LabelMap, extent );
     bool immutableLabel = false;
-    vtkKWEPaintbrushEnums::LabelType lastLabel = 
+    vtkKWEPaintbrushEnums::LabelType lastLabel =
                                   vtkKWEPaintbrushLabelData::NoLabelValue;
     while( !it1.IsAtEnd() )
-      { 
-      vtkKWEPaintbrushEnums::LabelType *inSI    = it1.BeginSpan();  
-      vtkKWEPaintbrushEnums::LabelType *inSI2   = it2.BeginSpan();  
+      {
+      vtkKWEPaintbrushEnums::LabelType *inSI    = it1.BeginSpan();
+      vtkKWEPaintbrushEnums::LabelType *inSI2   = it2.BeginSpan();
       vtkKWEPaintbrushEnums::LabelType *inSIEnd = it1.EndSpan();
       if( mutableLabelsPresent && !forceMutable )
         {
-        while (inSI != inSIEnd) 
+        while (inSI != inSIEnd)
           {
           if (*inSI2 != vtkKWEPaintbrushLabelData::NoLabelValue)
             {
@@ -436,11 +534,11 @@ int vtkKWEPaintbrushLabelData::Subtract( vtkKWEPaintbrushData * data, bool force
             }
           ++inSI;
           ++inSI2;
-          }      
+          }
         }
       else
         {
-        while (inSI != inSIEnd) 
+        while (inSI != inSIEnd)
           {
           if (*inSI2 != vtkKWEPaintbrushLabelData::NoLabelValue)
             {
@@ -449,7 +547,7 @@ int vtkKWEPaintbrushLabelData::Subtract( vtkKWEPaintbrushData * data, bool force
             }
           ++inSI;
           ++inSI2;
-          }      
+          }
         }
       it1.NextSpan();
       it2.NextSpan();
@@ -459,12 +557,21 @@ int vtkKWEPaintbrushLabelData::Subtract( vtkKWEPaintbrushData * data, bool force
   else if (vtkKWEPaintbrushStencilData *binaryPaintbrushData =
            vtkKWEPaintbrushStencilData::SafeDownCast(data))
     {
-    vtkImageStencilData *stencilData 
+    vtkImageStencilData *stencilData
       = binaryPaintbrushData->GetImageStencilData();
 
+
+    // Compute incremental deltas for Undo-Redo, if requested.
+    vtkSmartPointer< vtkImageStencilData > tempData = stencilData;
+    if (this->ComputeDelta)
+      {
+      tempData = vtkSmartPointer< vtkImageStencilData >::New();
+      tempData->DeepCopy(stencilData);
+      }
+
     int r1, r2, moreSubExtents, iter;
     bool immutableLabel = false;
-    vtkKWEPaintbrushEnums::LabelType lastLabel = 
+    vtkKWEPaintbrushEnums::LabelType lastLabel =
                                   vtkKWEPaintbrushLabelData::NoLabelValue;
     for (int z=extent[4]; z <= extent[5]; z++)
       {
@@ -474,12 +581,14 @@ int vtkKWEPaintbrushLabelData::Subtract( vtkKWEPaintbrushData * data, bool force
         moreSubExtents = 1;
         while( moreSubExtents )
           {
-          moreSubExtents = stencilData->GetNextExtent(
+          moreSubExtents = tempData->GetNextExtent(
             r1, r2, extent[0], extent[1], y, z, iter);
 
           // sanity check
           if (r1 <= r2)
             {
+
+            int removeStart = -1, removeEnd = -1;
             vtkKWEPaintbrushEnums::LabelType *beginExtent =
               static_cast<vtkKWEPaintbrushEnums::LabelType *>(this->LabelMap->GetScalarPointer(r1, y, z));
             vtkKWEPaintbrushEnums::LabelType *endExtent   =
@@ -502,29 +611,84 @@ int vtkKWEPaintbrushLabelData::Subtract( vtkKWEPaintbrushData * data, bool force
                   }
                 if( !immutableLabel )
                   {
-                  *beginExtent = vtkKWEPaintbrushLabelData::NoLabelValue;
+                  if (*beginExtent != vtkKWEPaintbrushLabelData::NoLabelValue)
+                    {
+                    *beginExtent = vtkKWEPaintbrushLabelData::NoLabelValue;
+
+                    // This scan line already exists on the data. Remove it from
+                    // the stroke
+                    if (this->ComputeDelta && removeStart != -1 && removeEnd == -1 && removeStart < r1)
+                      {
+                      stencilData->RemoveExtent( removeStart, r1-1, y, z );
+                      removeStart = removeEnd = -1; // reset them.
+                      }
+                    }
+                  else
+                    {
+                    if (removeStart == -1)
+                      {
+                      removeStart = r1;
+                      }
+                    }
                   }
+                else // this pixel is locked to any changes
+                  {
+                  if (removeStart == -1)
+                    {
+                    removeStart = r1;
+                    }
+                  }
+
                 ++beginExtent;
+                ++r1;
                 }
               }
             else
               {
               while (beginExtent <= endExtent)
                 {
-                *beginExtent = vtkKWEPaintbrushLabelData::NoLabelValue;
+
+                if (*beginExtent != vtkKWEPaintbrushLabelData::NoLabelValue)
+                  {
+                  *beginExtent = vtkKWEPaintbrushLabelData::NoLabelValue;
+
+                  // This scan line already exists on the data. Remove it from
+                  // the stroke
+                  if (this->ComputeDelta && removeStart != -1 && removeEnd == -1 && removeStart < r1)
+                    {
+                    stencilData->RemoveExtent( removeStart, r1-1, y, z );
+                    removeStart = removeEnd = -1; // reset them.
+                    }
+                  }
+                else
+                  {
+                  if (removeStart == -1)
+                    {
+                    removeStart = r1;
+                    }
+                  }
                 ++beginExtent;
+                ++r1;
                 }
               }
+
+            // This scan line already exists on the data. Remove it from the
+            // stroke
+            if (this->ComputeDelta && removeStart != -1 && removeEnd == -1)
+              {
+              stencilData->RemoveExtent( removeStart, r1 - 1, y, z );
+              }
+
             }
           } // end for each extent tuple
         } // end for each scan line
-      } // end of each slice 
+      } // end of each slice
     }
 
   this->LabelMap->Modified();
   this->Modified();
   return 1;
-} 
+}
 
 //----------------------------------------------------------------------------
 int vtkKWEPaintbrushLabelData::Replace( vtkKWEPaintbrushData * data, bool forceMutable )
@@ -533,39 +697,39 @@ int vtkKWEPaintbrushLabelData::Replace( vtkKWEPaintbrushData * data, bool forceM
   data->GetExtent(extentToBeReplaced);
   this->GetExtent(currentExtent);
 
-  if( !vtkKWEPaintbrushUtilities::GetIntersectingExtents( 
+  if( !vtkKWEPaintbrushUtilities::GetIntersectingExtents(
          currentExtent, extentToBeReplaced, extent ) )
     {
     // The stuff we are removing is entirely outside our bounds.
-    // Nothing to add. 
-    return 0; 
+    // Nothing to add.
+    return 0;
     }
 
-  // Handle Immubtability of sketches. 
+  // Handle Immubtability of sketches.
   // For details, see vtkKWEPaintbrushProperty::SetMubtale
   const bool mutableLabelsPresent = (this->ImmutableLabels.size() > 0);
-  vtkstd::set< vtkKWEPaintbrushEnums::LabelType >::const_iterator 
+  vtkstd::set< vtkKWEPaintbrushEnums::LabelType >::const_iterator
                     iterEnd = this->ImmutableLabels.end();
-  
 
-  if (vtkKWEPaintbrushLabelData *labelData = 
+
+  if (vtkKWEPaintbrushLabelData *labelData =
       vtkKWEPaintbrushLabelData::SafeDownCast(data))
     {
-    vtkImageIterator< vtkKWEPaintbrushEnums::LabelType > it2( 
+    vtkImageIterator< vtkKWEPaintbrushEnums::LabelType > it2(
                     labelData->GetLabelMap(), extent );
-    vtkImageIterator< vtkKWEPaintbrushEnums::LabelType > it1( 
+    vtkImageIterator< vtkKWEPaintbrushEnums::LabelType > it1(
                     this->LabelMap, extent );
     bool immutableLabel = false;
-    vtkKWEPaintbrushEnums::LabelType lastLabel = 
+    vtkKWEPaintbrushEnums::LabelType lastLabel =
                                   vtkKWEPaintbrushLabelData::NoLabelValue;
     while( !it1.IsAtEnd() )
-      { 
-      vtkKWEPaintbrushEnums::LabelType *inSI    = it1.BeginSpan();  
-      vtkKWEPaintbrushEnums::LabelType *inSI2   = it2.BeginSpan();  
+      {
+      vtkKWEPaintbrushEnums::LabelType *inSI    = it1.BeginSpan();
+      vtkKWEPaintbrushEnums::LabelType *inSI2   = it2.BeginSpan();
       vtkKWEPaintbrushEnums::LabelType *inSIEnd = it1.EndSpan();
       if( mutableLabelsPresent && !forceMutable )
         {
-        while (inSI != inSIEnd) 
+        while (inSI != inSIEnd)
           {
           // If this voxel is mutable..
           if( lastLabel != *inSI )
@@ -596,11 +760,11 @@ int vtkKWEPaintbrushLabelData::Replace( vtkKWEPaintbrushData * data, bool forceM
             }
           ++inSI;
           ++inSI2;
-          }      
+          }
         }
       else
         {
-        while (inSI != inSIEnd) 
+        while (inSI != inSIEnd)
           {
           if( *inSI2 == this->GetLabel() )
             {
@@ -615,7 +779,7 @@ int vtkKWEPaintbrushLabelData::Replace( vtkKWEPaintbrushData * data, bool forceM
             }
           ++inSI;
           ++inSI2;
-          }      
+          }
         }
       it1.NextSpan();
       it2.NextSpan();
@@ -624,12 +788,12 @@ int vtkKWEPaintbrushLabelData::Replace( vtkKWEPaintbrushData * data, bool forceM
   else if (vtkKWEPaintbrushStencilData *binaryPaintbrushData =
            vtkKWEPaintbrushStencilData::SafeDownCast(data))
     {
-    vtkImageStencilData *stencilData 
+    vtkImageStencilData *stencilData
       = binaryPaintbrushData->GetImageStencilData();
 
     int r1, r2, moreSubExtents, iter;
     bool immutableLabel = false;
-    vtkKWEPaintbrushEnums::LabelType lastLabel = 
+    vtkKWEPaintbrushEnums::LabelType lastLabel =
                                   vtkKWEPaintbrushLabelData::NoLabelValue;
     for (int z=extent[4]; z <= extent[5]; z++)
       {
@@ -695,13 +859,13 @@ int vtkKWEPaintbrushLabelData::Replace( vtkKWEPaintbrushData * data, bool forceM
             }
           } // end for each extent tuple
         } // end for each scan line
-      } // end of each slice 
+      } // end of each slice
     }
 
   this->LabelMap->Modified();
   this->Modified();
   return 1;
-} 
+}
 
 //----------------------------------------------------------------------------
 int vtkKWEPaintbrushLabelData::Clip( int extent[6] )
@@ -723,7 +887,7 @@ int vtkKWEPaintbrushLabelData::Clip( int extent[6] )
 
   bool remove = false, removed = false;
 
-  vtkKWEPaintbrushEnums::LabelType *ptr = 
+  vtkKWEPaintbrushEnums::LabelType *ptr =
     static_cast<vtkKWEPaintbrushEnums::LabelType *>(this->LabelMap->GetScalarPointer());
 
   for (idz=currentExtent[4]; idz<=currentExtent[5]; idz++)
@@ -815,14 +979,14 @@ unsigned long vtkKWEPaintbrushLabelData::GetMTime()
 }
 
 //----------------------------------------------------------------------------
-void vtkKWEPaintbrushLabelData::SetNoLabelValue( 
+void vtkKWEPaintbrushLabelData::SetNoLabelValue(
                   vtkKWEPaintbrushEnums::LabelType label )
 {
   vtkKWEPaintbrushLabelData::NoLabelValue = label;
 }
 
 //----------------------------------------------------------------------------
-void vtkKWEPaintbrushLabelData::SetMutable( 
+void vtkKWEPaintbrushLabelData::SetMutable(
     int isMutable, vtkKWEPaintbrushEnums::LabelType label )
 {
   if (isMutable)
@@ -835,6 +999,156 @@ void vtkKWEPaintbrushLabelData::SetMutable(
     }
 }
 
+//----------------------------------------------------------------------------
+int vtkKWEPaintbrushLabelData::IsMutable(
+    vtkKWEPaintbrushEnums::LabelType label )
+{
+  return (this->ImmutableLabels.find(label) == this->ImmutableLabels.end()) ? 0 : 1;
+}
+
+//----------------------------------------------------------------------------
+vtkKWEPaintbrushLabelData::LabelSetType vtkKWEPaintbrushLabelData::GetLabels()
+{
+  LabelSetType labels;
+  vtkDataArray * array = this->LabelMap->GetPointData()->GetScalars();
+  vtkKWEPaintbrushEnums::LabelType *arrayPointer =
+    static_cast<vtkKWEPaintbrushEnums::LabelType *>(array->GetVoidPointer(0));
+
+  const unsigned long size = array->GetDataSize();
+  for (unsigned long i = 0; i < size; ++i, ++arrayPointer)
+    {
+    vtkKWEPaintbrushEnums::LabelType l = *arrayPointer;
+    if (l != NoLabelValue)
+      {
+      labels.insert(l);
+      }
+    }
+
+  return labels;
+}
+
+//----------------------------------------------------------------------------
+void vtkKWEPaintbrushLabelData::RelabelDataToContiguousLabels()
+{
+  // Convenience method to collapse the labels in a label map. Consider a label
+  // map with labels of 32, 64, 125, 255 and a NoLabelValue of 0. This will
+  // collapse this data into labels of 1, 2, 3, 4.
+
+  // Build a map of the old (uncontiguous) labels to contiguous labels.
+  const LabelSetType oldLabels = this->GetLabels();
+  typedef vtkKWEPaintbrushEnums::LabelType LabelType;
+  typedef std::map< LabelType, LabelType > RelabelMapType;
+  RelabelMapType relabelMap;
+  LabelType i = 0;
+  for ( LabelSetType::const_iterator it = oldLabels.begin();
+      it != oldLabels.end(); ++it, ++i )
+    {
+    if (i == NoLabelValue)
+      {
+      ++i;
+      }
+    relabelMap.insert(std::pair< LabelType, LabelType >(*it, i));
+    }
+
+  // Check that the labels are really "uncontiguous". If they aren't, we have
+  // nothing to do.
+
+  int isContiguous = true;
+  for (RelabelMapType::const_iterator rit = relabelMap.begin();
+         (rit != relabelMap.end() && isContiguous); ++rit)
+    {
+    isContiguous &= (rit->first == rit->second);
+    }
+
+  // Now visit every label in the image and re-map it to its contiguous value.
+
+  if (!isContiguous)
+    {
+    vtkDataArray * array = this->LabelMap->GetPointData()->GetScalars();
+    vtkKWEPaintbrushEnums::LabelType *arrayPointer =
+      static_cast<vtkKWEPaintbrushEnums::LabelType *>(array->GetVoidPointer(0));
+
+    const unsigned long size = array->GetDataSize();
+    for (unsigned long i = 0; i < size; ++i, ++arrayPointer)
+      {
+      vtkKWEPaintbrushEnums::LabelType l = *arrayPointer;
+      if (l != NoLabelValue)
+        {
+        *arrayPointer = relabelMap[l];
+        }
+      }
+
+    this->LabelMap->Modified();
+    this->Modified();
+    }
+}
+
+//----------------------------------------------------------------------------
+void vtkKWEPaintbrushLabelData::SetLabelMap( vtkImageData * labelMap )
+{
+  // If the datatype of the label map is different from what is defined as
+  // vtkKWEPaintbrushEnums::GetLabelType(), we have to do a casting to that
+  // type.
+
+  if (this->LabelMap == NULL ||
+    this->LabelMap->GetScalarType() == vtkKWEPaintbrushEnums::GetLabelType())
+    {
+    vtkSetObjectBodyMacro( LabelMap, vtkImageData, labelMap );
+    }
+
+  // The scalar type is different....
+
+  // Check if labelMap's scalar range will fit in the our scalar type.
+
+  typedef std::numeric_limits< vtkKWEPaintbrushEnums::LabelType > LimitsType;
+  double range[2];
+  labelMap->GetScalarRange(range);
+
+  if ((range[0] >= LimitsType::min() && range[1] <= LimitsType::max()) ||
+      (range[1]-range[0]) > (LimitsType::max() - LimitsType::max()))
+    {
+
+    // Yes we fit.. just do regular casting..
+
+    vtkImageCast *cast = vtkImageCast::New();
+    cast->SetInput(labelMap);
+    cast->SetOutputScalarType(vtkKWEPaintbrushEnums::GetLabelType());
+    cast->Update();
+
+    vtkImageData *castedLabelMap = vtkImageData::New();
+    castedLabelMap->ShallowCopy(cast->GetOutput());
+    vtkSetObjectBodyMacro( LabelMap, vtkImageData, castedLabelMap );
+
+    cast->Delete();
+    castedLabelMap->Delete();
+    }
+
+  // We can't just cast simply. At least check if our scalar range fits
+  // within the precision we support.
+
+  else
+    {
+
+    // Shift to our range
+    vtkImageShiftScale *shiftScale = vtkImageShiftScale::New();
+    shiftScale->SetInput(labelMap);
+    shiftScale->SetOutputScalarType(vtkKWEPaintbrushEnums::GetLabelType());
+
+    // 1 offset assumes that NoLabelValue is 0.
+    shiftScale->SetShift(range[0]+1);
+    shiftScale->Update();
+
+    vtkImageData *shiftScaledLabelMap = vtkImageData::New();
+    shiftScaledLabelMap->ShallowCopy(shiftScale->GetOutput());
+    vtkSetObjectBodyMacro( LabelMap, vtkImageData, shiftScaledLabelMap );
+
+    shiftScale->Delete();
+    shiftScaledLabelMap->Delete();
+
+    }
+
+}
+
 //----------------------------------------------------------------------------
 void vtkKWEPaintbrushLabelData::PrintSelf(ostream& os, vtkIndent indent)
 {
@@ -846,7 +1160,7 @@ void vtkKWEPaintbrushLabelData::PrintSelf(ostream& os, vtkIndent indent)
     int extent[6];
     this->GetExtent(extent);
 
-    os << indent << "Extent: (" 
+    os << indent << "Extent: ("
        << extent[0] << ", "
        << extent[1] << ", "
        << extent[2] << ", "
@@ -857,12 +1171,12 @@ void vtkKWEPaintbrushLabelData::PrintSelf(ostream& os, vtkIndent indent)
     double spacing[3], origin[3];
     this->GetSpacing(spacing);
     this->GetOrigin(origin);
-    os << indent << "Spacing: (" 
+    os << indent << "Spacing: ("
        << spacing[0] << ", "
        << spacing[1] << ", "
        << spacing[2] << ")\n";
 
-    os << indent << "Origin: (" 
+    os << indent << "Origin: ("
        << origin[0] << ", "
        << origin[1] << ", "
        << origin[2] << ")\n";
