@@ -1,21 +1,21 @@
 //=============================================================================
 //   This file is part of VTKEdge. See vtkedge.org for more information.
 //
-//   Copyright (c) 2008 Kitware, Inc.
+//   Copyright (c) 2010 Kitware, Inc.
 //
-//   VTKEdge may be used under the terms of the GNU General Public License 
-//   version 3 as published by the Free Software Foundation and appearing in 
-//   the file LICENSE.txt included in the top level directory of this source
-//   code distribution. Alternatively you may (at your option) use any later 
-//   version of the GNU General Public License if such license has been 
-//   publicly approved by Kitware, Inc. (or its successors, if any).
+//   VTKEdge may be used under the terms of the BSD License
+//   Please see the file Copyright.txt in the root directory of
+//   VTKEdge for further information.
 //
-//   VTKEdge is distributed "AS IS" with NO WARRANTY OF ANY KIND, INCLUDING
-//   THE WARRANTIES OF DESIGN, MERCHANTABILITY, AND FITNESS FOR A PARTICULAR
-//   PURPOSE. See LICENSE.txt for additional details.
+//   Alternatively, you may see:
+//
+//   http://www.vtkedge.org/vtkedge/project/license.html
+//
+//
+//   For custom extensions, consulting services, or training for
+//   this or any other Kitware supported open source project, please
+//   contact Kitware at sales@kitware.com.
 //
-//   VTKEdge is available under alternative license terms. Please visit
-//   vtkedge.org or contact us at kitware@kitware.com for further information.
 //
 //=============================================================================
 #include "vtkKWEPaintbrushShapeEllipsoid.h"
@@ -32,13 +32,13 @@
 #include "vtkCellArray.h"
 #include "vtkKWEPaintbrushUtilities.h"
 
-#ifndef min 
+#ifndef min
 #define min(x,y) ((x<y) ? (x) : (y))
 #endif
 
 #define sign(x) ((x<0) ? (-1) : (1))
 
-vtkCxxRevisionMacro(vtkKWEPaintbrushShapeEllipsoid, "$Revision: 590 $");
+vtkCxxRevisionMacro(vtkKWEPaintbrushShapeEllipsoid, "$Revision: 3282 $");
 vtkStandardNewMacro(vtkKWEPaintbrushShapeEllipsoid);
 
 //----------------------------------------------------------------------
@@ -57,26 +57,26 @@ vtkKWEPaintbrushShapeEllipsoid::~vtkKWEPaintbrushShapeEllipsoid()
 }
 
 //----------------------------------------------------------------------
-vtkSmartPointer< vtkPolyData > 
-vtkKWEPaintbrushShapeEllipsoid::GetShapePolyData( 
+vtkSmartPointer< vtkPolyData >
+vtkKWEPaintbrushShapeEllipsoid::GetShapePolyData(
               double *center, vtkPlane *plane)
 {
- 
+
   if (plane == NULL)
     {
     // No Orientation specified. Return the whole Polydata. This is what will
     // be rendered on the volume widget
 
-    vtkParametricFunctionSource * parametricFunctionSource = 
+    vtkParametricFunctionSource * parametricFunctionSource =
       vtkParametricFunctionSource::New();
-    vtkParametricEllipsoid * ellipsoidFunction 
+    vtkParametricEllipsoid * ellipsoidFunction
                             = vtkParametricEllipsoid::New();
     parametricFunctionSource->SetParametricFunction(ellipsoidFunction);
     parametricFunctionSource->SetScalarModeToNone();
     parametricFunctionSource->GenerateTextureCoordinatesOff();
     parametricFunctionSource->SetUResolution( this->Resolution );
     parametricFunctionSource->Update();
-    
+
     vtkSmartPointer< vtkPolyData > pd = parametricFunctionSource->GetOutput();
 
     ellipsoidFunction->Delete();
@@ -84,13 +84,13 @@ vtkKWEPaintbrushShapeEllipsoid::GetShapePolyData(
 
     return pd;
     }
-  
+
   else
-  
+
     {
     double normal[3], origin[3], r[3];
-    r[0] = this->Width[0]/2.0; 
-    r[1] = this->Width[1]/2.0; 
+    r[0] = this->Width[0]/2.0;
+    r[1] = this->Width[1]/2.0;
     r[2] = this->Width[2]/2.0;
     plane->GetNormal(normal);
     plane->GetOrigin(origin);
@@ -106,25 +106,25 @@ vtkKWEPaintbrushShapeEllipsoid::GetShapePolyData(
         {
         return NULL;
         }
-      
-      vtkSmartPointer< vtkPolyData > templateOutline 
+
+      vtkSmartPointer< vtkPolyData > templateOutline
                   = vtkSmartPointer< vtkPolyData >::New();
 
       vtkPoints    *points      = vtkPoints::New();
       vtkCellArray *lines       = vtkCellArray::New();
       vtkIdType    *lineIndices = new vtkIdType[this->Resolution + 1];
-      
-      r[1] = r[1] * sqrt(1- 
+
+      r[1] = r[1] * sqrt(1-
           (origin[0]-center[0])*(origin[0]-center[0])/(r[0]*r[0]));
-      r[2] = r[2] * sqrt(1- 
+      r[2] = r[2] * sqrt(1-
           (origin[0]-center[0])*(origin[0]-center[0])/(r[0]*r[0]));
 
 
       for (int i = 0; i< this->Resolution; i++)
         {
-        const double angle = 2.0*vtkMath::Pi()* 
+        const double angle = 2.0*vtkMath::Pi()*
           static_cast<double>(i)/static_cast<double>(this->Resolution);
-        points->InsertPoint(static_cast<vtkIdType>(i), origin[0], 
+        points->InsertPoint(static_cast<vtkIdType>(i), origin[0],
                              center[1] + r[1] * cos(angle),
                              center[2] + r[2] * sin(angle));
         lineIndices[i] = static_cast<vtkIdType>(i);
@@ -148,17 +148,17 @@ vtkKWEPaintbrushShapeEllipsoid::GetShapePolyData(
         {
         return NULL;
         }
-      
-      vtkSmartPointer< vtkPolyData > templateOutline 
+
+      vtkSmartPointer< vtkPolyData > templateOutline
                   = vtkSmartPointer< vtkPolyData >::New();
 
       vtkPoints    *points      = vtkPoints::New();
       vtkCellArray *lines       = vtkCellArray::New();
       vtkIdType    *lineIndices = new vtkIdType[this->Resolution + 1];
-      
-      r[0] = r[0] * sqrt(1- 
+
+      r[0] = r[0] * sqrt(1-
           (origin[1]-center[1])*(origin[1]-center[1])/(r[1]*r[1]));
-      r[2] = r[2] * sqrt(1- 
+      r[2] = r[2] * sqrt(1-
           (origin[1]-center[1])*(origin[1]-center[1])/(r[1]*r[1]));
 
 
@@ -191,25 +191,25 @@ vtkKWEPaintbrushShapeEllipsoid::GetShapePolyData(
         {
         return NULL;
         }
-      
-      vtkSmartPointer< vtkPolyData > templateOutline 
+
+      vtkSmartPointer< vtkPolyData > templateOutline
                   = vtkSmartPointer< vtkPolyData >::New();
 
       vtkPoints    *points      = vtkPoints::New();
       vtkCellArray *lines       = vtkCellArray::New();
       vtkIdType    *lineIndices = new vtkIdType[this->Resolution + 1];
-      
-      r[0] = r[0] * sqrt(1- 
+
+      r[0] = r[0] * sqrt(1-
           (origin[1]-center[1])*(origin[1]-center[1])/(r[1]*r[1]));
-      r[2] = r[2] * sqrt(1- 
+      r[2] = r[2] * sqrt(1-
           (origin[1]-center[1])*(origin[1]-center[1])/(r[1]*r[1]));
 
 
       for (int i = 0; i< this->Resolution; i++)
         {
-        const double angle = 2.0*vtkMath::Pi()* 
+        const double angle = 2.0*vtkMath::Pi()*
           i/static_cast<double>(this->Resolution);
-        points->InsertPoint(static_cast<vtkIdType>(i), 
+        points->InsertPoint(static_cast<vtkIdType>(i),
                             center[0] + r[0] * cos(angle),
                             center[1] + r[1] * sin(angle),
                             origin[2]);
@@ -226,8 +226,8 @@ vtkKWEPaintbrushShapeEllipsoid::GetShapePolyData(
 
       return templateOutline;
       }
-    
-    else 
+
+    else
       {
       // TODO intersect cube with arbitrarily oriented plane and return polydata
       vtkErrorMacro( << "Not yet supported" );
@@ -241,30 +241,97 @@ vtkKWEPaintbrushShapeEllipsoid::GetShapePolyData(
 void vtkKWEPaintbrushShapeEllipsoid::GetStencil(
     vtkImageStencilData *stencilData, double p[3])
 {
-  int extent[6];
-  this->GetExtent( extent, p );
+  //int extent[6];
+  //this->GetExtent( extent, p );
 
-  // Create an elliposid image using vtkImageEllipsoidSource
-  // and convert that to a stencil..
-  vtkImageEllipsoidSource *ellipsoidImage = vtkImageEllipsoidSource::New(); 
-  ellipsoidImage->SetWholeExtent(extent);
-  ellipsoidImage->SetCenter((p[0] - this->Origin[0])/this->Spacing[0],
-                            (p[1] - this->Origin[1])/this->Spacing[1],
-                            (p[2] - this->Origin[2])/this->Spacing[2]);
-  ellipsoidImage->SetRadius(this->Width[0]/(2.0*this->Spacing[0]), 
-                            this->Width[1]/(2.0*this->Spacing[1]), 
-                            this->Width[2]/(2.0*this->Spacing[2]));
-  ellipsoidImage->SetOutputScalarTypeToUnsignedChar();
-  ellipsoidImage->SetInValue(255.0);
-  ellipsoidImage->SetOutValue(0.0);
-  ellipsoidImage->GetOutput()->Update();
-  ellipsoidImage->GetOutput()->SetSpacing(this->Spacing);
-  ellipsoidImage->GetOutput()->SetOrigin(this->Origin);
+  // A. Get an elliptical stencil
 
-  vtkKWEPaintbrushUtilities::GetStencilFromImage< 
-    vtkKWEPaintbrushUtilities::vtkFunctorGreaterThanEqualTo >( 
-      ellipsoidImage->GetOutput(), stencilData, 1.0);
-  ellipsoidImage->Delete();
+  // A.1. Compute extents that the stencil will have
+
+  double ellipsoidBounds[6];
+  int ellipsoidExtent[6];
+  for (int i = 0; i < 3; i++)
+    {
+    ellipsoidBounds[2*i] = p[i] - this->Width[i]/2.0;
+    ellipsoidBounds[2*i+1] = p[i] + this->Width[i]/2.0;
+    ellipsoidExtent[2*i] = static_cast< int >((ellipsoidBounds[2*i]-
+          this->Origin[i])/this->Spacing[i]);
+    ellipsoidExtent[2*i+1] = static_cast< int >((ellipsoidBounds[2*i+1]-
+          this->Origin[i])/this->Spacing[i] + 0.5);
+
+    // Clip the extents with the clip extent.
+    if (ellipsoidExtent[2*i] < this->ClipExtent[2*i])
+      {
+      ellipsoidExtent[2*i] = this->ClipExtent[2*i];
+      }
+    if (ellipsoidExtent[2*i+1] > this->ClipExtent[2*i+1])
+      {
+      ellipsoidExtent[2*i+1] = this->ClipExtent[2*i+1];
+      }
+    }
+
+  // A.2 Allocate the stencil
+
+  stencilData->SetExtent(ellipsoidExtent);
+  stencilData->SetSpacing(this->Spacing);
+  stencilData->SetOrigin(this->Origin);
+  stencilData->AllocateExtents();
+
+  // A.3 Fill the stencil with an ellipsoid
+
+  int xIdx, yIdx, zIdx, ends[2];
+  double startPt[3], inc[3];
+
+  for (int i = 0; i < 3; i++)
+    {
+    startPt[i] = 2.0 * (((double)ellipsoidExtent[2*i])*this->Spacing[i] +
+        this->Origin[i] - p[i])/this->Width[i];
+    inc[i] = 2.0 * this->Spacing[i] / this->Width[i];
+    }
+
+  double yInc = inc[1], xInc = inc[0], zInc = inc[2], zDist = startPt[2], xDist, yDist;
+  double zDistSq, zyDistSq, distSq;
+
+  // A.4 Rasterize
+
+  for (zIdx = ellipsoidExtent[4]; zIdx <= ellipsoidExtent[5]; zIdx++, zDist += zInc)
+    {
+    zDistSq = zDist * zDist;
+    yDist = startPt[1];
+    for (yIdx = ellipsoidExtent[2]; yIdx <= ellipsoidExtent[3]; yIdx++, yDist += yInc)
+      {
+      zyDistSq = zDistSq + (yDist * yDist);
+
+      xDist = startPt[0];
+      ends[0] = ends[1] = -1;
+
+      for (xIdx = ellipsoidExtent[0]; xIdx <= ellipsoidExtent[1]; xIdx++, xDist += xInc)
+        {
+        distSq = zyDistSq + (xDist * xDist);
+
+        if (ends[0] == -1 && distSq <= 1.0)
+          {
+          // found the start
+          ends[0] = xIdx;
+          continue;
+          }
+        if (ends[0] != -1 && distSq > 1.0)
+          {
+          // found the end, jump to to the next scan line. There can be only one
+          // segment per scan line in an ellipse.
+          ends[1] = xIdx;
+          stencilData->InsertNextExtent(ends[0], ends[1]-1, yIdx, zIdx);
+          break;
+          }
+        }
+
+      if (ends[1] == -1 && ends[0] != -1)
+        {
+        // scan line never completed.
+        stencilData->InsertNextExtent(ends[0], ellipsoidExtent[1], yIdx, zIdx);
+        }
+      }
+    }
 }
 
 //----------------------------------------------------------------------
@@ -303,20 +370,20 @@ int vtkKWEPaintbrushShapeEllipsoidFillBuffer( vtkKWEPaintbrushShapeEllipsoid * s
         double py = j * spacing[1] + origin[1] - p[1];
         double pz = k * spacing[2] + origin[2] - p[2];
 
-        const double distanceSq = px*px/r1square + 
-                                  py*py/r2square + 
+        const double distanceSq = px*px/r1square +
+                                  py*py/r2square +
                                   pz*pz/r3square;
-        
+
         if ( distanceSq > maxRadiusFactorSq )
-          {  
+          {
           // Outside the ellipse
           *np = static_cast< T >(0.0);
           continue;
           }
-        
+
         // Normalized distance of the point from the surface of the ellipse.
-        // This is 1.0 at the surface, 0.0 at the center, 2.0 at twice the 
-        // distance from the surface... 
+        // This is 1.0 at the surface, 0.0 at the center, 2.0 at twice the
+        // distance from the surface...
         const double distance = sqrt(distanceSq);
 
         if (distance <= (1.0 - transitionRegion))
@@ -330,7 +397,7 @@ int vtkKWEPaintbrushShapeEllipsoidFillBuffer( vtkKWEPaintbrushShapeEllipsoid * s
 
         //  Positive brush profile              Negarive brush profile
         //
-        //  
+        //
         // 255   -           /\                       ----        ----
         //                  /  \                          \      /
         // 127.5 -         /    \                          \    /
@@ -339,12 +406,12 @@ int vtkKWEPaintbrushShapeEllipsoidFillBuffer( vtkKWEPaintbrushShapeEllipsoid * s
         //
         //               |    |    |                      |   |   |
         //             -2r    0    2r                   -2r   0   2r
-        //             
+        //
         // value = state ? (255.0 - 127.5 * distance) : 127.5 * distance;
 
 
         // Don't use 0. So we shrink the range within the shape by 1 on
-        // each end. 1 - 254. The reason is 0 is used to indicate an 
+        // each end. 1 - 254. The reason is 0 is used to indicate an
         // outside value (outside the brush). See vtkKWEPaintbrushGrayscaleData
         //
         // clamp value [1 - 254]
@@ -357,11 +424,11 @@ int vtkKWEPaintbrushShapeEllipsoidFillBuffer( vtkKWEPaintbrushShapeEllipsoid * s
           value = 254.0;
           }
 
-        *np = static_cast< T >(value); 
+        *np = static_cast< T >(value);
         }
       }
     }
-  
+
   return 1;
 }
 
@@ -371,17 +438,17 @@ void vtkKWEPaintbrushShapeEllipsoid::GetGrayscaleData(
 {
   // Compute the extents of the an image centered about p.
   int extent[6];
-  this->GetExtent( extent, p ); 
+  this->GetExtent( extent, p );
   imageData->SetSpacing(this->Spacing);
   imageData->SetOrigin(this->Origin);
   imageData->SetExtent(extent);
-  
+
   imageData->SetScalarType(this->GetScalarType());
   imageData->AllocateScalars();
 
   switch (imageData->GetScalarType())
     {
-    vtkTemplateMacro( vtkKWEPaintbrushShapeEllipsoidFillBuffer( 
+    vtkTemplateMacro( vtkKWEPaintbrushShapeEllipsoidFillBuffer(
                 this, imageData, static_cast< VTK_TT >(0), extent, p ));
     }
 }
@@ -402,7 +469,7 @@ void vtkKWEPaintbrushShapeEllipsoid::SetWidth( double newWidthX, double newWidth
 //----------------------------------------------------------------------
 void vtkKWEPaintbrushShapeEllipsoid::SetWidth( double newWidth[3] )
 {
-  this->SetWidth( newWidth[0], newWidth[1], newWidth[2] );  
+  this->SetWidth( newWidth[0], newWidth[1], newWidth[2] );
 }
 
 //----------------------------------------------------------------------
@@ -411,39 +478,39 @@ int vtkKWEPaintbrushShapeEllipsoid::Resize(double d[3], int ResizeType)
   // If the user specified a constraint on the resize type, use that,
   // otherwise default to whatever the widget told us in the functions'
   // argument.
-  const int resizeType = (this->ResizeConstraint == 
+  const int resizeType = (this->ResizeConstraint ==
       PaintbrushResizeUnConstrained) ? ResizeType : this->ResizeConstraint;
 
-  // Define a minimum size that the shape will take. The shape will not 
+  // Define a minimum size that the shape will take. The shape will not
   // get smaller than this.
   const double minSize = 0.5;
- 
+
   double newWidth[3] = { this->Width[0], this->Width[1], this->Width[2] };
 
   if (resizeType == vtkKWEPaintbrushShape::PaintbrushResizeAnisotropic)
     {
-     
+
     // non-isotropic resize. This will resize each axis according to the
     // factor specified along each axis.
-    
+
     for (unsigned int i=0; i<3; i++)
       {
-      if (d[i] > 0.0 || this->Width[i] > minSize) 
+      if (d[i] > 0.0 || this->Width[i] > minSize)
         {
         newWidth[i] *= (1+d[i]/10.0);
         }
-      }    
+      }
     }
-  
+
   else
     {
 
     // Not an AnIsotropic resize.. This will resize each axis by the same
-    // factor. This factor will be the norm of the factor vector specified 
+    // factor. This factor will be the norm of the factor vector specified
     // as the functions' argument
-    
+
     // Calculate the sign.. (grow or shrink)
-    
+
     unsigned int idx = 0;
     double max = fabs(d[0]);
     int signVal;
@@ -484,14 +551,14 @@ int vtkKWEPaintbrushShapeEllipsoid::Resize(double d[3], int ResizeType)
 
   // Make sure we aren't smaller than the minimum
   if (newWidth[0] < minSize ||
-      newWidth[1] < minSize || 
+      newWidth[1] < minSize ||
       newWidth[2] < minSize)
     {
-    return 0;    
+    return 0;
     }
 
   // Now change our size to the new size.
-  this->SetWidth( newWidth );  
+  this->SetWidth( newWidth );
   return 1;
 }
 
@@ -512,7 +579,7 @@ void vtkKWEPaintbrushShapeEllipsoid::DeepCopy(vtkKWEPaintbrushShape *s)
       }
     this->Resolution = sb->Resolution;
     }
-  
+
   this->Superclass::DeepCopy(s);
   this->Modified();
 }
@@ -520,7 +587,7 @@ void vtkKWEPaintbrushShapeEllipsoid::DeepCopy(vtkKWEPaintbrushShape *s)
 //----------------------------------------------------------------------
 void vtkKWEPaintbrushShapeEllipsoid::GetAnnotation(char *s)
 {
-  sprintf(s, "Diameter: (%0.3g,%0.3g,%0.3g)", 
+  sprintf(s, "Diameter: (%0.3g,%0.3g,%0.3g)",
       this->Width[0], this->Width[1], this->Width[2]);
 }
 
